%========================================================================
%Input : (1) imorg  	: The reference image
%        (2) imdist 	: The distorted image 
%		 (3) reflev 	: Number of levels for wavelet decomposition
%		 (4) block_size	: Size of the blocks in the decoupling stage
%		 (5) doh		: distance over vertical height of the high resolution image   	
%Output: (1) MIQE te visual information fidelity measure between the two images

%Default Usage:
%   Given 2 test images img1 and img2, whose dynamic range is 0-255
%
%   miqeVal = miqe(img1, img2, 5, 3, 2);
% Note : Decoupling and Mutual Informations stages we performed as in VIF
%========================================================================
function [miqeVal]=miqe(imorg,imdist,reflev,block_size,doh)

if (nargin < 3)
  reflev = 4; %murat
  block_size = 4; %murat
  doh = 2; %murat
elseif(nargin < 4)
  block_size = 3;
  doh = 2;
elseif(nargin < 5)
  doh = 2;  
end

% Non-overlapping blocksize in Gaussian Scale Mixture (GSM) model
M = block_size;
subbands=1:3*reflev+1;
sigma_nsq=0.4;
num = zeros(1,subbands);
den = zeros(1,subbands); 


%1) Pixel value to physical luminance
% display device black-level offset (sRGB)
b = 0;
% display device pixel-value-to-voltage gain (sRGB)
k = 0.02874;
% display device gamma (sRGB)
gam = 2.2;
% initialize the pixel-value-to-luminance lookup table
pixel_vals = -255:255;
pix2lum_table =abs((b + k*pixel_vals)).^gam;
for row = 1 : size(imorg,1)     
    imorg(row,:) = pix2lum_table(imorg(row,:)+256);
end
for row = 1 : size(imdist,1)     
    imdist(row,:) = pix2lum_table(imdist(row,:)+256);
end

% 2) 9/7 Discrete wavelet decomposition. 
% [imorg, imdist] = getCsf(imorg,imdist);
ref = dwt2d_norm(imorg,reflev,'97');
[rowSz,colSz] = size(imorg);
[distRowSz,distColSz] = size(imdist);
scl = log2(rowSz/distRowSz);
rowLevSz = rowSz / (2^(reflev));
colLevSz = colSz / (2^(reflev)); 
ind = 1;

% Reference Image Wavelet Decomposition
for kk =1:reflev
    if (kk == 1)
        orgArr{ind} = ref(1:rowLevSz,1:colLevSz);
        ind = ind +1;
    end
    orgArr{ind}     = ref(rowLevSz+1:2*rowLevSz,1:colLevSz);
    orgArr{ind + 1} = ref(rowLevSz+1:2*rowLevSz,colLevSz+1:2*colLevSz);
    orgArr{ind + 2} = ref(1:rowLevSz,colLevSz+1:2*colLevSz);
    ind = ind + 3;
    rowLevSz = 2*rowLevSz;
    colLevSz = 2*colLevSz;
end

% Distance over height adjustment to obtain same visual angle  
if(size(imdist,1) == size(imorg,1))
    test = double(dwt2d_norm(imdist,reflev,'97'));
    subbands_dist = subbands;
else
    test = double(dwt2d_norm(imdist,reflev-1,'97'));
    doh = 2^(scl) * doh;
    subbands_dist = subbands(1:end-scl*3);
end

rowLevSz = rowSz / (2^(reflev));
colLevSz = colSz / (2^(reflev)); 
ind = 1;

% Distorted Image Wavelet Decomposition
for kk =1:reflev
    if (kk == 1)
        distArr{ind} = test(1:rowLevSz,1:colLevSz);
        ind = ind +1;
    end
    if( (size(imdist,1) == size(imorg,1)) || (kk < reflev))
        distArr{ind}     = test(rowLevSz+1:2*rowLevSz,1:colLevSz);
        distArr{ind + 1} = test(rowLevSz+1:2*rowLevSz,colLevSz+1:2*colLevSz);
        distArr{ind + 2} = test(1:rowLevSz,colLevSz+1:2*colLevSz);
    else
        distArr{ind}     = zeros(rowLevSz,colLevSz);
        distArr{ind + 1} = zeros(rowLevSz,colLevSz);
        distArr{ind + 2} = zeros(rowLevSz,colLevSz);        
    end
    ind = ind + 3;
    rowLevSz = 2*rowLevSz;
    colLevSz = 2*colLevSz;
end


% 5) Contrast Sensitivity Function and Subband Size Compensator
a = 0.31; 
b = 0.69; 
c = 0.29;
fs = rowSz/2;
la_vec = [];
for ii = reflev : -1 : 1
    if(ii == reflev)
        la_vec = [la_vec ii*ones(1,4)];%lambda vector for each subband
    else
        la_vec = [la_vec ii*ones(1,3)];
    end
end
subband = 1:3*reflev+1;

or_ind = [2:3:3*reflev+1 4:3:3*reflev+1];
or =-ones(1,3*reflev+1);
or(or_ind) = 1;
F_lambda = (pi * fs * doh)./(180 * 2.^(la_vec(subband)));
w =  F_lambda ./(0.85 + 0.15 * or);
H = (a+b*w).* exp(-c*w);  % CSF value for each subband kang li

rouse = 1:3*reflev+1;
sW = 2.^((reflev - ceil((rouse -1)./3))*2);
sWH = sW.*(H);


% 3) Decoupling
% Calculate the noise and the attenuation parameters of the distortion channel
[Att,noise]=vifsub_est_m(orgArr,distArr,subbands_dist,M);

% Calculate the parameters of Gaussian Scale Mixture model for the reference image
[ssarr, larr, cuarr]=refparams_vecgsm(orgArr,subbands,M);

% reorder subbands. This is needed since the outputs of the above functions
% are not in the same order
noise_temp=cell(1,max(subbands_dist));
att_temp=noise_temp;
for(kk=1:length(subbands_dist))
    noise_temp{subbands_dist(kk)}=noise{kk};
    att_temp{subbands_dist(kk)}=Att{kk};
end

% 4) Mutual Information
% compute reference and distorted image information from each subband
for i=1:length(subbands)
    sub=subbands(i);
    if(sub <= length(subbands_dist))    
        A=att_temp{sub};
        W=noise_temp{sub};
    end
    ss=ssarr{sub};
    lambda = larr(sub,:);
    cu=cuarr{sub};
    
    % compute the size of the window used in the distortion channel estimation, and use it to calculate the offset from subband borders
    % we do this to avoid all coefficients that may suffer from boundary
    % effects
    lev=ceil((sub-1)/6);
    winsize=2^lev+1; offset=(winsize-1)/2;
    offset=ceil(offset/M);
    
    
    % select only valid portion of the output.
%     if(sub <= length(subbands_dist)) //murat
%         A=A(offset+1:end-offset,offset+1:end-offset);
%         W=W(offset+1:end-offset,offset+1:end-offset);
%     end
%     ss=ss(offset+1:end-offset,offset+1:end-offset);
    
    temp1=0; temp2=0;
    for j=1:length(lambda)
        if(sub <= length(subbands_dist))
            temp1=temp1+sum(sum((log2(1+A.*A.*ss.*lambda(j)./(W+sigma_nsq))))); % distorted image information for the i'th subband
        end
        temp2=temp2+sum(sum((log2(1+ss.*lambda(j)./(sigma_nsq))))); % reference image information
    end
    num(i)=temp1;
    den(i)=temp2;
end

% 6) Pooling (MIQE) 
miqeVal = sum(sWH.*num)./sum(sWH.*den);
